var CandyShop = (function(self) { return self; }(CandyShop || {}));

CandyShop.Paint = (function(self, Candy, $) {
	
	var _ctx,
	_lastX,
	_lastY,
	_mousePressed = false,
	_brushSize = 3,
	_brushColor = '#000000';
	_bgColor = '#ffffff';
	_saveActionUrl = '/candy-plugins/paint/saveimage.php';
	_chatUrl = window.location.protocol + '//' + window.location.hostname;
	
	/* init */
	self.init = function(numColors) {
	    
	    self.applyTranslations();
	    
	    var html = '<li id="paint-control" data-tooltip="' + $.i18n._('candyshopPaintMessagecolor') + '"></li>';
	    
	    $('#emoticons-icon').after(html);
	    
	    $('#paint-control').click(function(event) {
		CandyShop.Paint.showPainter(this);
	    });
	    
	};
	
	/* showPainter */
	self.showPainter = function(elem) {
	    elem = $(elem);
	    var pos = elem.offset(),
	    menu = $('#context-menu'),
	    content = $('ul', menu),
	    paint = '<canvas id="paintCanvas" width="300" height="300" style="border:1px solid #ddd;"></canvas>',
	    tools = '<div class="paint-tools"><span class="paint-submit" title="submit"></span><span class="paint-clear" title="clear"></span></div>',
	    i;
	    
	    $('#tooltip').hide();
	    $('#context-menu').addClass('right-bottom');
	    $('#context-menu .arrow-top').hide();
	    $('#context-menu .arrow-bottom').removeAttr('style');
	    
	    if ( $('#paintCanvas').size() == 0 ) {
		
		content.html('<li class="paint">' + tools + paint + '</li>');
		self.InitDraw();
		
		var posLeft = Candy.Util.getPosLeftAccordingToWindowBounds(menu, pos.left),
		    posTop  = Candy.Util.getPosTopAccordingToWindowBounds(menu, pos.top);
		
		menu.css({'left': posLeft.px, 'top': posTop.px, backgroundPosition: posLeft.backgroundPositionAlignment + ' ' + posTop.backgroundPositionAlignment});
		
		$('#context-menu .paint-tools .paint-submit').bind('click',self.submitPaint);
		$('#context-menu .paint-tools .paint-clear').bind('click',self.clearPaint);
		
	    }
	    
	    menu.show();
	    
	    return true;
	};
	
	/* InitDraw */
	self.InitDraw = function() {
	    _ctx = document.getElementById('paintCanvas').getContext("2d");
	    self.canvasFill();
	    $('#paintCanvas').mousedown(function(e) {
		_mousePressed = true;
		self.Draw(e.pageX - $(this).offset().left, e.pageY - $(this).offset().top, false);
	    });

	    $('#paintCanvas').mousemove(function(e) {
		if (_mousePressed) {
		    self.Draw(e.pageX - $(this).offset().left, e.pageY - $(this).offset().top, true);
		}
	    });

	    $('#paintCanvas').mouseup(function(e) {
		if (_mousePressed) {
		    _mousePressed = false;
		}
	    });

	    $('#paintCanvas').mouseleave(function(e) {
		if (_mousePressed) {
		    _mousePressed = false;
		}
	    });
	    //self.drawImage(backgroundImage);
	}
	
	/* canvasFill */
	self.canvasFill = function(color){
	    if ( typeof color == 'undefined' ) var color = _bgColor;
	    _ctx = document.getElementById('paintCanvas').getContext("2d");
	    _ctx.beginPath();
	    _ctx.moveTo(0,0);
	    _ctx.lineTo(_ctx.canvas.width,0);
	    _ctx.lineTo(_ctx.canvas.width,_ctx.canvas.height);
	    _ctx.lineTo(0,_ctx.canvas.width);
	    _ctx.fillStyle = color;
	    _ctx.closePath();
	    _ctx.fill();
	}
	
	/* Draw */
	self.Draw = function(x, y, isDown) {
	    if (isDown) {
		_ctx.beginPath();
		_ctx.strokeStyle = _brushColor;
		_ctx.lineWidth = _brushSize;
		_ctx.lineJoin = "round";
		_ctx.moveTo(_lastX, _lastY);
		_ctx.lineTo(x, y);
		_ctx.closePath();
		_ctx.stroke();
	    }
	    _lastX = x;
	    _lastY = y;
	}
	
	/* drawImage */
	self.drawImage = function(image_url) {
	    var image = new Image();
	    image.src = image_url;
	    $(image).load(function() {
		_ctx.drawImage(image, 0, 0, canvasWidth, canvasHeight);
	    });
	}
	
	/* clearPaint */
	self.clearPaint = function(){
	    
	    _ctx.setTransform(1, 0, 0, 1, 0, 0);
	    _ctx.clearRect(0, 0, _ctx.canvas.width, _ctx.canvas.height);
	    self.canvasFill();
	    
	}
	
	/* submitPaint */
	self.submitPaint = function(){
	    
	    var strDataURI = _ctx.canvas.toDataURL(); 
	    
	    if (!strDataURI) {
		alert("Sorry, this browser is not capable of saving files!");
		return false;
	    }
	    
	    $.ajax({
		type: "POST",
		url: _saveActionUrl,
		data: { 
		   img: strDataURI
		}
	    }).done(function(img_url) {
		
		if(!!img_url){
		    var room = Candy.View.getCurrent();
		    Candy.Core.Action.Jabber.Room.Message(room.roomJid, _chatUrl + '/candy-plugins/paint/images/' + img_url, 'groupchat');
		}
		
	    });
	    
	}
	
	/* set */
	self.set = function(name,value){
	    
	    if( name == 'saveActionUrl' ) _saveActionUrl = value;
	    if( name == 'chatUrl' ) _chatUrl = value;
	    
	}
	
	/* applyTranslations */
	self.applyTranslations = function() {
	    var translations = {
	      'en' : 'Paint',
	      'ru' : 'Нарисовать картинку',
	      'de' : '',
	      'fr' : '',
	      'nl' : '',
	      'es' : ''
	    };
	    $.each(translations, function(k, v) {
		if(Candy.View.Translation[k]) {
		    Candy.View.Translation[k].candyshopPaintMessagecolor = v;
		}
	    });
	};
	
	return self;
    
}(CandyShop.Paint || {}, Candy, jQuery));
